//*****************************************************************************
//*File:		QUEUE.H
//*Purpose: Provides a 'C object oriented implementation of a circular queue.
//*			The queue is meant to be accessed using 4 functions:
//*			QueueInit - Must be called to before any other queue function.
//*			GetWord	- Gets a word from the queue and returns the queue status
//*							Returns QUEUE_EMPTY if there is no data in the queue
//*			PutWord	- Puts a word into the queue and returns the queue status
//*							Returns QUEUE_FULL if there is no room to store data into queue.
//*			GetQueueStatus - Returns the current queue status
//*Author:	Steve Bell
//*Date:		10/5/00
//*****************************************************************************
#ifndef _QUEUE_H
#define _QUEUE_H


typedef unsigned int WORD;

typedef enum
{
	QUEUE_FULL = 0,
	QUEUE_EMPTY,
	QUEUE_IN_USE
} QueueStatusType;


typedef struct
{
	WORD *wordArray;
	WORD *wordPutPtr;
   WORD *wordGetPtr;
   WORD *wordEndPtr;
	QueueStatusType Status;
} QueueType;


#ifdef _QUEUE_C
//*****************************************************************
//Function:	QueueInit
//Purpose:	Initializes all queue parameters.  It must be called before
//				queue is used!!!!  May be used to reset the queue.
//Parameters:	None
//Returns:		None
//****************************************************************
BOOLEAN QueueInit(UINT32 queueSize);

//*****************************************************************
//Function:	PutWord
//Purpose:	To allow the caller to check the queue's status
//				This function does not modify status.  It simply returns
//				the status stored in the private queue.
//Parameters:	None
//Returns:		QueueStatus - the queue status
//****************************************************************
QueueStatusType GetQueueStatus( void );

//*****************************************************************
//Function:	PutWord
//Purpose:	If there is room in the queue, stores the word in the queue
//				If the queue is full, the function returns
//Parameters:	WORD wordInput - word to store
//Returns:		QueueStatus - the queue status; either QUEUE_IN_USE or QUEUE_FULL
//****************************************************************
QueueStatusType PutWord( WORD wordInput );

//*****************************************************************
//Function:	GetWord
//Purpose:	If there is data in the queue, returns the next word
//				If the queue is empty, the function returns the QueueStatus
//Parameters:	WORD *wordOutput - ptr where word is returned
//Returns:		QueueStatus - the queue status; either QUEUE_IN_USE or
//										QUEUE_EMPTY
//****************************************************************
QueueStatusType GetWord( WORD *wordOutput );


//*****************************************************************
//Function:	PeakQueue
//Purpose:	If there is data in the queue, returns the next word
//              but leaves the queue intact
//				If the queue is empty, the function returns the QueueStatus
//Parameters:	WORD *wordOutput - ptr where word is returned
//Returns:		QueueStatus - the queue status; either QUEUE_IN_USE or
//										QUEUE_EMPTY
//****************************************************************
QueueStatusType PeakQueue( WORD *wordOutput );

//*****************************************************************
//Function:	GetNumberOfWords
//Purpose:	Returns the number of words in queue
//Parameters:	none
//Returns:	Number of words in Queue -- 0 if empty		
//****************************************************************
unsigned long GetNumberOfWords(void);
unsigned long GetCapacity(void);

#else

//*****************************************************************
//Function:	QueueInit
//Purpose:	Initializes all queue parameters.  It must be called before
//				queue is used!!!!  May be used to reset the queue.
//Parameters:	None
//Returns:		None
//****************************************************************
extern BOOLEAN QueueInit(UINT32 queueSize);

//*****************************************************************
//Function:	PutWord
//Purpose:	To allow the caller to check the queue's status
//				This function does not modify status.  It simply returns
//				the status stored in the private queue.
//Parameters:	None
//Returns:		QueueStatus - the queue status
//****************************************************************
extern QueueStatusType GetQueueStatus( void );

//*****************************************************************
//Function:	PutWord
//Purpose:	If there is room in the queue, stores the word in the queue
//				If the queue is full, the function returns
//Parameters:	WORD wordInput - word to store
//Returns:		QueueStatus - the queue status; either QUEUE_IN_USE or QUEUE_FULL
//****************************************************************
extern QueueStatusType PutWord( WORD wordInput );

//*****************************************************************
//Function:	GetWord
//Purpose:	If there is data in the queue, returns the next word
//				If the queue is empty, the function returns the QueueStatus
//Parameters:	WORD *wordOutput - ptr where word is returned
//Returns:		QueueStatus - the queue status; either QUEUE_IN_USE or
//										QUEUE_EMPTY
//****************************************************************
extern QueueStatusType GetWord( WORD *wordOutput );

//*****************************************************************
//Function:	PeakQueue
//Purpose:	If there is data in the queue, returns the next word
//              but leaves the queue intact
//				If the queue is empty, the function returns the QueueStatus
//Parameters:	WORD *wordOutput - ptr where word is returned
//Returns:		QueueStatus - the queue status; either QUEUE_IN_USE or
//										QUEUE_EMPTY
//****************************************************************
extern QueueStatusType PeakQueue( WORD *wordOutput );

//*****************************************************************
//Function:	GetNumberOfWords
//Purpose:	If there is data in the queue, returns the next word
//              but leaves the queue intact
//				If the queue is empty, the function returns the QueueStatus
//Parameters:	WORD *wordOutput - ptr where word is returned
//Returns:	Number of words in Queue -- 0 if empty		
//****************************************************************
extern unsigned long GetNumberOfWords(void);
extern unsigned long GetCapacity(void);

#endif

#endif //End-of-file queue.h
